# Program heatdiffxx.py
# version 2010-02-22
# Calculates heat diffusion from outside into a room
#  with internal intermittent heat source
# Outside temperature from file, or sine or square wave
# requires python interpreter (www.python.org)
# Appends results to weather file
# command line example:
# >python heatdiffxx.py parametersfile.txt [weather_file column_index] 
#
#    heatdiffxx.py is this script
#    the parameters file must not contain empty rows
#    it may contain comment lines starting with #
#    the file name must not contain spaces
#    the file path should have forward slashes: path/to/file/name
#    this works from windows98 and later
# Weather file layout: space delimited, any date format
#  must be hourly data
#  the last command line parameter is index of temperature column
#  counting from zero
# This program file imports file TSlices.py
#   which contains class definitions
# 
# Output format:
# output file name is [parametersfile].out.txt
#
# programmer: Tim Padfield<tim@padfield.dk>
# this program is open source, under GPLv2 licence. 
# It may freely be copied and altered
# but please describe your alterations in this comment section.
# layout of parameters file is in an example file


import sys, os, re, math

#-----IMPORT SLICE class defs>-------#

from TSlices import *

#----- OPEN PARAMETERS (INPUT) FILE ---------#

inputSuccess = 0

if len(sys.argv) > 1:
  try:
    startparamsfile = sys.argv[1]
    pfile = open(startparamsfile, 'r')
    inputSuccess = 1
  except:
    print 'Something wrong, try again'

if inputSuccess == 0:
  print 'needs parameters file'
  success = 0
  while success == 0:
     try:
       print 'enter path and filename:',
       startparamsfile = raw_input()
       pfile = open(startparamsfile, 'r')
       success = 1
     except:
       print "Can't open  parameters file, try again"

#-------OPEN [OPTIONAL] WEATHER FILE--------------#

tFromFile = 0
if len(sys.argv) > 3: # temperature from file
    try:
       weatherfile = sys.argv[2]
       wFile = open(weatherfile, 'r')
       tIndex = sys.argv[3]
       tIndex = int(tIndex)
       tFromFile = 1

#--MAKE CONTAINER FILE FOR WEATHER FILE WITH APPENDED WALL T--#
       wOutFile = weatherfile.split('.')
       wOutFile = wOutFile[0] + 'PlusWall.txt'
       wOutFile = open(wOutFile, 'w')
       print wOutFile
    except:
       print 'Something wrong with temperature file, try again'



#--------- OPEN OUTPUT FILE --------------#

inFileName = startparamsfile.split('/')
inFileName = inFileName[-1]
inFileName = inFileName.split('.')
inFileName = inFileName[0]
outFileName = inFileName + '.out.txt'
ofile = open(outFileName,'w')
print ' '
print 'OUTPUT FILE ', outFileName #testing
print ' '
print 'STARTING CONDITIONS:'
print ' '

#------ READ STARTING PARAMETERS FROM FILE-------#

ss = [] #make empty list of slice objects
sNumber = 0 #slice number (not type)

for line in pfile:
   pr = line.split(';')
   if pr[0] == '#': continue
   #change items 2 to last into floating point
   try:
      for a in range(2,len(pr)):
         if pr[a] == '':
            pr[a] = 0.0
         pr[a] = float(pr[a])
   except: pass
   if pr[0] == '0': # weather
      # construct Weather object
      # pr[1] is an optional comment
      s0 = TWeather(sNumber,pr[2],pr[3],pr[4],pr[5],pr[6],pr[7])
      ss.append(s0)
      sNumber += 1
      s0.showStatus()

   if pr[0] == '1': # construct wall slice
      s1 = Wall(sNumber,pr[2],pr[3],pr[4],pr[5],pr[6])
      ss.append(s1)
      ss[sNumber].showStatus()
      sNumber += 1

   if pr[0] == '2': # construct room slice
      s1 = Room(sNumber,pr[2],pr[3],pr[4],pr[5],pr[6],pr[7],pr[8])
      ss.append(s1)
      ss[sNumber].showStatus()
      sNumber += 1

#-----READ IN GLOBAL VARIABLES--------#

   if pr[0] == '9': # global variables list
      repRate = pr[2] #calculate every x seconds
      stopAt = pr[3] #stop at hour
      linesPerHour = pr[4] #output lines per hour
      tIncrement = repRate/3600 # calc. cycle in hours
      writeToFile = linesPerHour/tIncrement # when to write output
      if writeToFile < 1: writeToFile = 1 #but not between calc. cycles!
      print 'GLOBALS: ', 'repeat rate=',repRate,' stop at hour=',stopAt,
      print ' lines per hour=',linesPerHour
      print ' ' 

   

#---- ALL START DATA IN PLACE - SET TIME AND WEATHER ----#

elapsedTime = 0 # hours from start of run
writeToFileCounter = 10000000 # forces results to print at time zero
numberOfSlices = len(ss)
lastSlice = numberOfSlices-1
ss[0].update(stopAt,repRate,elapsedTime) #make weather, first arg is dummy
print 'number of slices = ', numberOfSlices

#---- START CALCULATION ----#

if tFromFile:
   commentLine = 1
   while commentLine:
      nextLine = wFile.readline()
      print nextLine #testing
      if nextLine[0] == '#': continue
      commentLine = 0
      wCols = nextLine.split(' ')
      wT = float(wCols[tIndex])    
      ss[0].wT = wT
      dummyLine = '#' + nextLine
      wOutFile.write(dummyLine) #just to keep synchrony at beginning

#------- MAIN LOOP ---------#

while elapsedTime < stopAt :
  for s in range(1,numberOfSlices):
    ss[s].diffuse(elapsedTime,repRate,ss[s-1]) #diffuse calls update for previous slice
  ss[lastSlice].update(0,repRate,elapsedTime) # update last slice with no flow to next slice

  # is it time to write to file?
  elapsedTime += tIncrement
  writeToFileCounter += 1
  if writeToFileCounter > writeToFile:
    writeToFileCounter = 0
    #compose output and print to file and to screen
    oLine = str('%3d' % elapsedTime)
    for s in range (0,numberOfSlices):
      oLine += str(' % 3.1f' % (ss[s].tt))
    #jouleContent = 0
    #for s in range(3,numberOfSlices):
    #  jouleContent += ss[s].joules
    #oLine += str(' %3.4f' % (jouleContent))
    ofile.write(oLine)
    ofile.write('\n')
    #print oLine    

    if tFromFile:
      try:
        #write input line with appended wall data
        appendedLine = nextLine[:-2] + ' ' + oLine + ' \n'
        wOutFile.write(appendedLine)
        print appendedLine,
        nextLine = wFile.readline()
        wCols = nextLine.split(' ')
        wT = float(wCols[tIndex])
        ss[0].wT = wT
      except:
        print 'Probably end of weather file'

# ---- END OF MAIN LOOP -----#

ofile.close()
pfile.close()
sys.exit(1)

